@@ -17,9 +17,14 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
-import sge
+import os
 import random
 
+import sge
+
+
+DATA = os.path.join(os.path.dirname(__file__), "data")
+
 
 class Game(sge.Game):
     def event_key_press(self, key, char):
@@ -36,9 +41,10 @@ class Game(sge.Game):
 
 class Circle(sge.Object):
     def __init__(self, x, y):
-        super(Circle, self).__init__(x, y, 1, sprite='circle',
-              collision_precise=True, image_blend=sge.Color("red"),
-              image_alpha=128)
+        super(Circle, self).__init__(x, y, 1, sprite=circle_sprite,
+                                     collision_precise=True,
+                                     image_blend=sge.Color("red"),
+                                     image_alpha=128)
 
     def event_step(self, time_passed, delta_mult):
         left_key = 'left'
@@ -72,13 +78,16 @@ class Circle(sge.Object):
 
 
 def main():
+    global circle_sprite
+
     # Create Game object
     Game(width=240, height=240, scale_smooth=True,
          collision_events_enabled=False)
 
     # Load sprites
-    sge.Sprite('circle', width=32, height=32, origin_x=16, origin_y=16)
-    fence = sge.Sprite('fence')
+    circle_sprite = sge.Sprite('circle', DATA, width=32, height=32,
+                               origin_x=16, origin_y=16)
+    fence = sge.Sprite('fence', DATA)
 
     # Load backgrounds
     layers = (sge.BackgroundLayer(fence, 0, 0, 0),)
@@ -92,7 +101,8 @@ def main():
     views = [sge.View(0, 0, 0, 0, 240, 240, 240, 240)]
 
     # Create rooms
-    sge.Room(objects, 640, 640, views=views, background=background)
+    sge.game.start_room = sge.Room(objects, 640, 640, views=views,
+                                   background=background)
 
     sge.game.start()
 
