@@ -123,9 +123,9 @@ class Game(object):
 
     .. attribute:: window_icon
 
-       The name of the image file located in one of the directories
-       specified in :data:`sge.image_directories` to use as the window
-       icon.  If set to :const:`None`, the SGE chooses the icon.
+       The path to the image file to use as the window icon.  If set
+       to :const:`None`, the SGE chooses the icon.  If the file
+       specified does not exist, :exc:`IOError` is raised.
 
     .. attribute:: collision_events_enabled
 
@@ -264,14 +264,12 @@ class Game(object):
     def window_icon(self, value):
         r.game_window_icon = value
         if value is not None:
-            for path in sge.image_directories:
-                try:
-                    image = pygame.image.load(os.path.join(path, value))
-                except pygame.error:
-                    continue
-                else:
-                    pygame.display.set_icon(image)
-                    break
+            try:
+                image = pygame.image.load(value)
+            except pygame.error as e:
+                raise IOError(e)
+            else:
+                pygame.display.set_icon(image)
 
     def __init__(self, width=640, height=480, fullscreen=False, scale=None,
                  scale_proportional=True, scale_smooth=False, fps=60,
@@ -529,19 +527,6 @@ class Game(object):
                     del self.current_room.alarms[a]
                     self.current_room.event_alarm(a)
 
-                # Transition
-                rd = self.current_room.rd
-                if rd["t_update"] is not None:
-                    rd["t_time_passed"] += real_time_passed
-
-                    if rd["t_time_passed"] < rd["t_duration"]:
-                        complete = rd["t_time_passed"] / rd["t_duration"]
-                        rd["t_update"](self.current_room, complete)
-                        rd["t_complete_last"] = complete
-                        self.project_sprite(rd["t_sprite"], 0, 0, 0)
-                    else:
-                        rd["t_update"] = None
-
                 # Step events
                 self.event_step(real_time_passed, delta_mult)
                 self.current_room.event_step(real_time_passed, delta_mult)
@@ -593,17 +578,32 @@ class Game(object):
                     obj.xprevious = obj.x
                     obj.yprevious = obj.y
 
+                # Transition
+                rd = self.current_room.rd
+                if rd["t_update"] is not None:
+                    rd["t_time_passed"] += real_time_passed
+
+                    if rd["t_time_passed"] < rd["t_duration"]:
+                        complete = rd["t_time_passed"] / rd["t_duration"]
+                        rd["t_update"](self.current_room, complete)
+                        rd["t_complete_last"] = complete
+                        self.project_sprite(rd["t_sprite"], 0, 0, 0)
+                    else:
+                        rd["t_update"] = None
+
                 # Refresh
                 self.refresh()
-
-            self.event_game_end()
-            pygame.quit()
-            sge.game = None
+            else:
+                self.event_game_end()
+                pygame.quit()
+                sge.game = None
         else:
             raise AttributeError("sge.game.start_room is undefined.")
 
     def end(self):
         """Properly end the game."""
+        if self.current_room is not None:
+            self.current_room.event_room_end()
         r.game_running = False
 
     def pause(self, sprite=None):
@@ -623,9 +623,8 @@ class Game(object):
         with them.
         """
         if sprite is None:
-            sge.image_directories.append(os.path.dirname(__file__))
             try:
-                sprite = sge.Sprite("_pygame_sge_pause")
+                sprite = sge.Sprite("pause", os.path.dirname(__file__))
             except IOError:
                 font = sge.Font("Droid Sans", size=24)
                 sprite = sge.Sprite(width=320, height=240)
@@ -633,8 +632,6 @@ class Game(object):
                                  halign=sge.ALIGN_CENTER,
                                  valign=sge.ALIGN_MIDDLE)
 
-            del sge.image_directories[-1]
-
         r.game_paused = True
 
         while r.game_paused and r.game_running:
@@ -967,7 +964,8 @@ class Game(object):
                     images.append((img, x, y, layer.z, None))
 
             for obj in self.current_room.objects:
-                if obj is not self.mouse and obj.sprite is not None:
+                if (obj.visible and obj.sprite is not None and
+                        obj is not self.mouse):
                     img = s_get_image(obj.sprite, obj.image_index,
                                       obj.image_xscale, obj.image_yscale,
                                       obj.image_rotation, obj.image_alpha,
