@@ -21,8 +21,20 @@ from __future__ import print_function
 from __future__ import unicode_literals
 
 import pygame
+import six
 
 import sge
+from sge import r
+from sge.r import (_get_dot_sprite, _get_line_sprite, _get_rectangle_sprite,
+                   _get_ellipse_sprite, _get_circle_sprite,
+                   _get_polygon_sprite, _get_text_sprite,
+                   o_update_collision_lists, r_reset, r_update_fade,
+                   r_update_dissolve, r_update_pixelate, r_update_wipe_left,
+                   r_update_wipe_right, r_update_wipe_up, r_update_wipe_down,
+                   r_update_wipe_upleft, r_update_wipe_upright,
+                   r_update_wipe_downleft, r_update_wipe_downright,
+                   r_update_wipe_matrix, r_update_iris_in, r_update_iris_out,
+                   s_get_image)
 
 
 __all__ = ['Room']
@@ -30,14 +42,11 @@ __all__ = ['Room']
 
 class Room(object):
 
-    """Class for rooms.
-
+    """
     This class stores the settings and objects found in a room.  Rooms
     are used to create separate parts of the game, such as levels and
     menu screens.
 
-    Every game must have at least one room.
-
     .. attribute:: width
 
        The width of the room in pixels.  If set to :const:`None`,
@@ -64,32 +73,27 @@ class Room(object):
 
        The vertical position of the background in the room.
 
+    .. attribute:: alarms
+
+       A dictionary containing the alarms of the room.  Each value
+       decreases by 1 each frame (adjusted for delta timing if it is
+       enabled).  When a value is at or below 0,
+       :meth:`sge.Room.event_alarm` is executed with ``alarm_id`` set to
+       the respective key, and the item is deleted from this dictionary.
+
     .. attribute:: objects
 
        A list containing all :class:`sge.Object` objects in the
        room.  (Read-only)
 
-    .. attribute:: objects_by_class
-
-       A dictionary of lists containing all :class:`sge.Object`
-       objects in the room, separated by class.  The dictionary keys are
-       classes that have been registered with
-       :meth:`sge.Game.register_class`, and the lists contain only
-       those objects which are instances of the class indicated by the
-       respective key.  (Read-only)
-
-    .. attribute:: room_number
-
-       The index of this room in the game, where ``0`` is the first
-       room.  (Read-only)
+    .. attribute:: rd
 
+       Reserved dictionary for internal use by the SGE.  (Read-only)
     """
 
     def __init__(self, objects=(), width=None, height=None, views=None,
-                 background=None, background_x=0, background_y=0,
-                 room_number=None):
-        """Constructor method.
-
+                 background=None, background_x=0, background_y=0):
+        """
         Arguments:
 
         - ``views`` -- A list containing all :class:`sge.View` objects
@@ -99,251 +103,238 @@ class Room(object):
         - ``background`` -- The :class:`sge.Background` object used.  If
           set to :const:`None`, a new background will be created with no
           layers and the color set to black.
-        - ``room_number`` -- The position in :data:`sge.game.rooms` to
-          insert this room into.  If set to :const:`None`, it will be
-          appended to the end of the list.
 
         All other arguments set the respective initial attributes of the
         room.  See the documentation for :class:`sge.Room` for more
         information.
-
         """
+        self.rd = {}
         self.width = width if width is not None else sge.game.width
         self.height = height if height is not None else sge.game.height
-        self._start_width = self.width
-        self._start_height = self.height
+        self.rd["swidth"] = self.width
+        self.rd["sheight"] = self.height
         self.background_x = background_x
         self.background_y = background_y
-        self._destroyed = False
-        self._new_objects = []
-        self._projections = []
-
-        self._alarms = {}
+        self.alarms = {}
+        self.__new_objects = []
+        self.rd["projections"] = []
 
         if views is not None:
             self.views = list(views)
         else:
             self.views = [sge.View(0, 0)]
-        self._start_views = []
+        self.rd["sviews"] = []
 
-        self._view_start_x = {}
-        self._view_start_y = {}
-        self._view_start_xport = {}
-        self._view_start_yport = {}
-        self._view_start_width = {}
-        self._view_start_height = {}
+        self.rd["view_sx"] = {}
+        self.rd["view_sy"] = {}
+        self.rd["view_sxport"] = {}
+        self.rd["view_syport"] = {}
+        self.rd["view_swidth"] = {}
+        self.rd["view_sheight"] = {}
 
         if background is not None:
             self.background = background
         else:
             self.background = sge.Background((), sge.Color("black"))
-        self._start_background = self.background
+        self.rd["sbackground"] = self.background
+        self.rd["sbackground_x"] = self.background_x
+        self.rd["sbackground_y"] = self.background_y
 
-        rooms = sge.game.rooms[:]
-        if room_number is None or room_number >= len(rooms):
-            self.room_number = len(rooms)
-            rooms.append(self)
-        else:
-            self.room_number = room_number
-            rooms.insert(room_number, self)
-        sge.game.rooms = rooms
-
-        self._started = False
-        self._has_started = False
+        self.rd["started"] = False
+        self.__has_started = False
 
         self.objects = []
-        self.objects_by_class = {}
-        for cls in sge.game.registered_classes:
-            self.objects_by_class[cls] = []
 
         self.add(sge.game.mouse)
         for obj in objects:
             self.add(obj)
-        self._start_objects = []
+        self.rd["sobjects"] = []
 
-        self._object_start_x = {}
-        self._object_start_y = {}
-        self._object_start_z = {}
-        self._object_start_sprite = {}
-        self._object_start_visible = {}
-        self._object_start_checks_collisions = {}
-        self._object_start_tangible = {}
-        self._object_start_bbox_x = {}
-        self._object_start_bbox_y = {}
-        self._object_start_bbox_width = {}
-        self._object_start_bbox_height = {}
-        self._object_start_collision_ellipse = {}
-        self._object_start_collision_precise = {}
+        self.rd["object_sx"] = {}
+        self.rd["object_sy"] = {}
+        self.rd["object_sz"] = {}
+        self.rd["object_ssprite"] = {}
+        self.rd["object_svisible"] = {}
+        self.rd["object_schecks_collisions"] = {}
+        self.rd["object_stangible"] = {}
+        self.rd["object_sbbox_x"] = {}
+        self.rd["object_sbbox_y"] = {}
+        self.rd["object_sbbox_width"] = {}
+        self.rd["object_sbbox_height"] = {}
+        self.rd["object_scollision_ellipse"] = {}
+        self.rd["object_scollision_precise"] = {}
 
         if self.views:
-            self._collision_area_size = max(int(self.views[0].width / 10),
-                                            int(self.views[0].height / 10))
+            size = max(int(self.views[0].width / 10),
+                       int(self.views[0].height / 10))
         else:
-            self._collision_area_size = sge.COLLISION_AREA_SIZE_DEFAULT
+            size = sge.COLLISION_AREA_SIZE_DEFAULT
 
-        size = self._collision_area_size
-        self._collision_areas = []
-        for i in range(0, self.width, size):
-            column = [[] for j in range(0, self.height, size)]
-            self._collision_areas.append(column)
+        self.rd["collision_area_size"] = size
+        self.rd["collision_areas"] = []
+        for i in six.moves.range(0, self.width, size):
+            column = [[] for j in six.moves.range(0, self.height, size)]
+            self.rd["collision_areas"].append(column)
 
         # The "Void" is the area outside the room.  This area is
         # infinite, so everything in the Void is checked against
         # everything else in the Void (otherwise an infinite grid, which
         # is impossible, would be needed).
-        self._collision_area_void = []
+        self.rd["collision_area_void"] = []
 
     def add(self, obj):
-        """Add a Object object to the room.
+        """
+        Add an object to the room.
 
         Arguments:
 
         - ``obj`` -- The :class:`sge.Object` object to add.
 
         """
-        if not isinstance(obj, sge.Object):
-            obj = sge.game.objects[obj]
-
+        obj.alive = True
         if obj not in self.objects:
             objects = self.objects[:]
             objects.append(obj)
             self.objects = objects
 
-            objects_by_class = self.objects_by_class.copy()
-            for cls in objects_by_class:
-                if isinstance(obj, cls):
-                    objects = objects_by_class[cls][:]
-                    objects.append(obj)
-                    objects_by_class[cls] = objects
-            self.objects_by_class = objects_by_class
-
-            if self is sge.game.current_room and self._started:
+            if self is sge.game.current_room and self.rd["started"]:
                 obj.event_create()
             else:
-                self._new_objects.append(obj)
+                self.__new_objects.append(obj)
 
-    def start(self):
-        """Start the room.
-
-        If the room has been changed, reset it to its original state.
+            o_update_collision_lists(obj)
 
+    def remove(self, obj):
         """
-        if self._has_started:
-            self._reset()
+        Remove an object from the room.
 
-        self.resume()
-
-    def resume(self):
-        """Continue the room from where it left off.
-
-        If the room is unchanged (e.g. has not been started yet), this
-        method behaves in the same way that :meth:`sge.Room.start` does.
+        Arguments:
 
+        - ``obj`` -- The :class:`sge.Object` object to remove.
         """
+        objects = self.objects[:]
+        while obj in objects:
+            objects.remove(obj)
+        self.objects = objects
+
+        o_update_collision_lists(obj)
+        if self is sge.game.current_room:
+            obj.event_destroy()
+
+    def start(self, transition=None, transition_time=1500):
+        """
+        Start the room, resetting to its original state if it has been
+        started previously.
+
+        Arguments:
+
+        - ``transition`` -- The type of transition to use.  Should be
+          one of the following:
+
+          - :const:`None` (no transition)
+          - ``"fade"`` (fade to black)
+          - ``"dissolve"``
+          - ``"pixelate"``
+          - ``"wipe_left"`` (wipe right to left)
+          - ``"wipe_right"`` (wipe left to right)
+          - ``"wipe_up"`` (wipe bottom to top)
+          - ``"wipe_down"`` (wipe top to bottom)
+          - ``"wipe_upleft"`` (wipe bottom-right to top-left)
+          - ``"wipe_upright"`` (wipe bottom-left to top-right)
+          - ``"wipe_downleft"`` (wipe top-right to bottom-left)
+          - ``"wipe_downright"`` (wipe top-left to bottom-right)
+          - ``"wipe_matrix"``
+          - ``"iris_in"``
+          - ``"iris_out"``
+
+          If an unsupported value is given, default to :const:`None`.
+
+        - ``transition_time`` -- The time the transition should take in
+          milliseconds.  Has no effect if ``transition`` is
+          :const:`None`.
+        """
+        if self.__has_started:
+            r_reset(self)
+
+        self.resume(transition, transition_time)
+
+    def resume(self, transition=None, transition_time=1500):
+        """
+        Start the room without resetting to its original state if it has
+        been started previously.  See the documentation for
+        :meth:`sge.Room.start` for more information.
+        """
+        transitions = {
+            "fade": r_update_fade, "dissolve": r_update_dissolve,
+            "pixelate": r_update_pixelate, "wipe_left": r_update_wipe_left,
+            "wipe_right": r_update_wipe_right, "wipe_up": r_update_wipe_up,
+            "wipe_down": r_update_wipe_down,
+            "wipe_upleft": r_update_wipe_upleft,
+            "wipe_upright": r_update_wipe_upright,
+            "wipe_downleft": r_update_wipe_downleft,
+            "wipe_downright": r_update_wipe_downright,
+            "wipe_matrix": r_update_wipe_matrix,
+            "iris_in": r_update_iris_in, "iris_out": r_update_iris_out}
+
+        if transition in transitions and transition_time > 0:
+            self.rd["t_update"] = transitions[transition]
+            self.rd["t_sprite"] = sge.Sprite.from_screenshot()
+            self.rd["t_duration"] = transition_time
+            self.rd["t_time_passed"] = 0
+            self.rd["t_complete_last"] = 0
+            self.rd["t_matrix_remaining"] = None
+        else:
+            self.rd["t_update"] = None
+
+        sge.game.unpause()
         sge.game.current_room = self
 
-        if not self._has_started:
-            self._start_objects = self.objects[:]
+        if not self.__has_started:
+            self.rd["sobjects"] = self.objects[:]
             for obj in self.objects:
-                self._object_start_x[obj.id] = obj.x
-                self._object_start_y[obj.id] = obj.y
-                self._object_start_z[obj.id] = obj.z
-                self._object_start_sprite[obj.id] = obj.sprite
-                self._object_start_visible[obj.id] = obj.visible
-                self._object_start_checks_collisions[obj.id] = obj.checks_collisions
-                self._object_start_tangible[obj.id] = obj.tangible
-                self._object_start_bbox_x[obj.id] = obj.bbox_x
-                self._object_start_bbox_y[obj.id] = obj.bbox_y
-                self._object_start_bbox_width[obj.id] = obj.bbox_width
-                self._object_start_bbox_height[obj.id] = obj.bbox_height
-                self._object_start_collision_ellipse[obj.id] = obj.collision_ellipse
-                self._object_start_collision_precise[obj.id] = obj.collision_precise
+                self.rd["object_sx"][id(obj)] = obj.x
+                self.rd["object_sy"][id(obj)] = obj.y
+                self.rd["object_sz"][id(obj)] = obj.z
+                self.rd["object_ssprite"][id(obj)] = obj.sprite
+                self.rd["object_svisible"][id(obj)] = obj.visible
+                self.rd["object_schecks_collisions"][id(obj)] = obj.checks_collisions
+                self.rd["object_stangible"][id(obj)] = obj.tangible
+                self.rd["object_sbbox_x"][id(obj)] = obj.bbox_x
+                self.rd["object_sbbox_y"][id(obj)] = obj.bbox_y
+                self.rd["object_sbbox_width"][id(obj)] = obj.bbox_width
+                self.rd["object_sbbox_height"][id(obj)] = obj.bbox_height
+                self.rd["object_scollision_ellipse"][id(obj)] = obj.collision_ellipse
+                self.rd["object_scollision_precise"][id(obj)] = obj.collision_precise
 
-            self._start_views = self.views[:]
+            self.rd["sviews"] = self.views[:]
             for view in self.views:
-                self._view_start_x[id(view)] = view.x
-                self._view_start_y[id(view)] = view.y
-                self._view_start_xport[id(view)] = view.xport
-                self._view_start_yport[id(view)] = view.yport
-                self._view_start_width[id(view)] = view.width
-                self._view_start_height[id(view)] = view.height
+                self.rd["view_sx"][id(view)] = view.x
+                self.rd["view_sy"][id(view)] = view.y
+                self.rd["view_sxport"][id(view)] = view.xport
+                self.rd["view_syport"][id(view)] = view.yport
+                self.rd["view_swidth"][id(view)] = view.width
+                self.rd["view_sheight"][id(view)] = view.height
 
-        if not self._started:
+        for obj in self.objects:
+            obj.rd["collision_areas"] = []
+            o_update_collision_lists(obj)
+
+        started = self.rd["started"]
+        self.rd["started"] = True
+        self.__has_started = True
+        if not started:
             self.event_room_start()
             for obj in self.objects:
                 obj.event_create()
         else:
-            while self._new_objects:
-                self._new_objects[0].event_create()
-                del self._new_objects[0]
+            while self.__new_objects:
+                self.__new_objects[0].event_create()
+                del self.__new_objects[0]
             self.event_room_resume()
 
-        self._started = True
-        self._has_started = True
-
-    def set_alarm(self, alarm_id, value):
-        """Set an alarm.
-
-        After this method is called, ``value`` will reduce by 1 each
-        frame (adjusted for delta timing if it is enabled) until it
-        reaches 0, at which point :meth:`sge.Room.event_alarm` will be
-        executed with ``alarm_id``.
-
-        See the documentation for :meth:`sge.Game.set_alarm` for more
-        information.
-
-        """
-        if value is not None:
-            self._alarms[alarm_id] = value
-        elif alarm_id in self._alarms:
-            del self._alarms[alarm_id]
-
-    def get_alarm(self, alarm_id):
-        """Return the value of an alarm.
-
-        See the documentation for :meth:`sge.Game.get_alarm` for more
-        information.
-
-        """
-        if alarm_id in self._alarms:
-            return self._alarms[alarm_id]
-        else:
-            return None
-
-    def end(self, next_room=None, resume=True):
-        """End the current room.
-
-        Arguments:
-
-        - ``next_room`` -- The room number of the room to go to next.
-          If set to :const:`None`, the room after this one is chosen.
-        - ``resume`` -- Whether or not to resume the next room instead
-          of restarting it.
-
-        If the room chosen as the next room does not exist, the game is
-        ended.
-
-        This triggers this room's :meth:`sge.Room.event_room_end` and
-        resets the state of this room.
-
-        """
-        self.event_room_end()
-        self._reset()
-
-        if next_room is None:
-            next_room = self.room_number + 1
-
-        if (next_room >= -len(sge.game.rooms) and
-                next_room < len(sge.game.rooms)):
-            if resume:
-                sge.game.rooms[next_room].resume()
-            else:
-                sge.game.rooms[next_room].start()
-        else:
-            sge.game.end()
-
     def project_dot(self, x, y, z, color):
-        """Project a single-pixel dot onto the room.
+        """
+        Project a single-pixel dot onto the room.
 
         Arguments:
 
@@ -355,18 +346,18 @@ class Room(object):
 
         See the documentation for :meth:`sge.Sprite.draw_dot` for more
         information.
-
         """
         if not isinstance(color, sge.Color):
             e = "`{}` is not a sge.Color object.".format(repr(color))
             raise TypeError(e)
 
-        sprite = sge.game._get_dot_sprite(color)
+        sprite = _get_dot_sprite(color)
         self.project_sprite(sprite, 0, x, y, z)
 
     def project_line(self, x1, y1, x2, y2, z, color, thickness=1,
                      anti_alias=False):
-        """Project a line segment onto the room.
+        """
+        Project a line segment onto the room.
 
         Arguments:
 
@@ -382,7 +373,6 @@ class Room(object):
 
         See the documentation for :meth:`sge.Sprite.draw_line` for more
         information.
-
         """
         if not isinstance(color, sge.Color):
             e = "`{}` is not a sge.Color object.".format(repr(color))
@@ -396,13 +386,13 @@ class Room(object):
         x2 -= x
         y2 -= y
 
-        sprite = sge.game._get_line_sprite(x1, y1, x2, y2, color, thickness,
-                                           anti_alias)
+        sprite = _get_line_sprite(x1, y1, x2, y2, color, thickness, anti_alias)
         self.project_sprite(sprite, 0, x, y, z)
 
     def project_rectangle(self, x, y, z, width, height, fill=None,
                           outline=None, outline_thickness=1):
-        """Project a rectangle onto the room.
+        """
+        Project a rectangle onto the room.
 
         Arguments:
 
@@ -414,7 +404,6 @@ class Room(object):
 
         See the documentation for :meth:`sge.Sprite.draw_rectangle` for
         more information.
-
         """
         if fill is not None and not isinstance(fill, sge.Color):
             e = "`{}` is not a sge.Color object.".format(repr(fill))
@@ -428,13 +417,14 @@ class Room(object):
         draw_y = outline_thickness // 2
         x -= draw_x
         y -= draw_y
-        sprite = sge.game._get_rectangle_sprite(width, height, fill, outline,
-                                                outline_thickness)
+        sprite = _get_rectangle_sprite(width, height, fill, outline,
+                                       outline_thickness)
         self.project_sprite(sprite, 0, x, y, z)
 
     def project_ellipse(self, x, y, z, width, height, fill=None,
                         outline=None, outline_thickness=1, anti_alias=False):
-        """Project an ellipse onto the room.
+        """
+        Project an ellipse onto the room.
 
         Arguments:
 
@@ -451,7 +441,6 @@ class Room(object):
 
         See the documentation for :meth:`sge.Sprite.draw_ellipse` for
         more information.
-
         """
         if fill is not None and not isinstance(fill, sge.Color):
             e = "`{}` is not a sge.Color object.".format(repr(fill))
@@ -465,13 +454,14 @@ class Room(object):
         draw_y = outline_thickness // 2
         x -= draw_x
         y -= draw_y
-        sprite = sge.game._get_ellipse_sprite(width, height, fill, outline,
-                                              outline_thickness, anti_alias)
+        sprite = _get_ellipse_sprite(width, height, fill, outline,
+                                     outline_thickness, anti_alias)
         self.project_sprite(sprite, 0, x, y, z)
 
     def project_circle(self, x, y, z, radius, fill=None, outline=None,
                        outline_thickness=1, anti_alias=False):
-        """Project a circle onto the room.
+        """
+        Project a circle onto the room.
 
         Arguments:
 
@@ -483,7 +473,6 @@ class Room(object):
 
         See the documentation for :meth:`sge.Sprite.draw_circle` for
         more information.
-
         """
         if fill is not None and not isinstance(fill, sge.Color):
             e = "`{}` is not a sge.Color object.".format(repr(fill))
@@ -492,12 +481,48 @@ class Room(object):
             e = "`{}` is not a sge.Color object.".format(repr(outline))
             raise TypeError(e)
 
-        sprite = sge.game._get_circle_sprite(radius, fill, outline,
-                                             outline_thickness, anti_alias)
+        sprite = _get_circle_sprite(radius, fill, outline, outline_thickness,
+                                    anti_alias)
         self.project_sprite(sprite, 0, x - radius, y - radius, z)
 
+    def project_polygon(self, points, z, fill=None, outline=None,
+                        outline_thickness=1, anti_alias=False):
+        """
+        Draw a polygon on the sprite.
+
+        Arguments:
+
+        - ``points`` -- A list of points relative to the room to
+          position each of the polygon's angles.  Each point should be a
+          tuple in the form ``(x, y)``, where x is the horizontal
+          location and y is the vertical location.
+        - ``z`` -- The Z-axis position of the projection in the room.
+
+        See the documentation for :meth:`sge.Sprite.draw_polygon` for
+        more information.
+        """
+        if fill is not None and not isinstance(fill, sge.Color):
+            e = "`{}` is not a sge.Color object.".format(repr(fill))
+            raise TypeError(e)
+        if outline is not None and not isinstance(outline, sge.Color):
+            e = "`{}` is not a sge.Color object.".format(repr(outline))
+            raise TypeError(e)
+
+        xlist = []
+        ylist = []
+        for point in points:
+            xlist.append(point[0])
+            ylist.append(point[1])
+        x = min(xlist)
+        y = min(ylist)
+
+        sprite = _get_polygon_sprite(points, fill, outline, outline_thickness,
+                                     anti_alias)
+        self.project_sprite(sprite, 0, x, y, z)
+
     def project_sprite(self, sprite, image, x, y, z, blend_mode=None):
-        """Project a sprite onto the room.
+        """
+        Project a sprite onto the room.
 
         Arguments:
 
@@ -509,20 +534,17 @@ class Room(object):
 
         See the documentation for :meth:`sge.Sprite.draw_sprite` for
         more information.
-
         """
-        if not isinstance(sprite, sge.Sprite):
-            sprite = sge.game.sprites[sprite]
-
-        img = sprite._get_image(image)
+        img = s_get_image(sprite, image)
         x -= sprite.origin_x
         y -= sprite.origin_y
-        self._projections.append((img, x, y, z, blend_mode))
+        self.rd["projections"].append((img, x, y, z, blend_mode))
 
     def project_text(self, font, text, x, y, z, width=None, height=None,
                     color=sge.Color("black"), halign=sge.ALIGN_LEFT,
                     valign=sge.ALIGN_TOP, anti_alias=True):
-        """Project text onto the room.
+        """
+        Project text onto the room.
 
         Arguments:
 
@@ -534,117 +556,73 @@ class Room(object):
 
         See the documentation for :meth:`sge.Sprite.draw_text` for more
         information.
-
         """
         if not isinstance(color, sge.Color):
             e = "`{}` is not a sge.Color object.".format(repr(color))
             raise TypeError(e)
 
-        sprite = sge.game._get_text_sprite(font, text, width, height, color,
-                                           halign, valign, anti_alias)
+        sprite = _get_text_sprite(font, text, width, height, color, halign,
+                                  valign, anti_alias)
         self.project_sprite(sprite, 0, x, y, z)
 
-    def move(self, room_number):
-        """Move the room.
-
-        Arguments:
-
-        - ``room_number`` -- The new position in :data:`sge.game.rooms` to
-          insert this room into.
-
-        """
-        rooms = sge.game.rooms[:]
-        if not self._destroyed and len(rooms) < self.room_number:
-            del rooms[self.room_number]
-            self.room_number = room_number
-            rooms.insert(room_number, self)
-            sge.game.rooms = rooms
-
-    def destroy(self):
-        """Destroy the room.
-
-        .. note::
-
-           If the room is being used, it will not be completely
-           destroyed until this use stops.
-
-        """
-        rooms = sge.game.rooms[:]
-        if not self._destroyed and len(rooms) < self.room_number:
-            self._destroyed = True
-            del rooms[self.room_number]
-            sge.game.rooms = rooms
-
     def event_room_start(self):
-        """Room start event.
-
-        Called when the room starts.  It is always called after any game
-        start events and before any object create events occurring at
-        the same time.
-
+        """
+        Called when the room starts for the first time, and when it is
+        restarted with :meth:`sge.Room.start`.  It is always called
+        after any :meth:`sge.Game.event_game_start` and before any
+        :class:`sge.Object.event_create` occurring at the same time.
         """
         pass
 
     def event_room_resume(self):
-        """Room resume event.
-
-        Called when the room resumes without being reset to its original
-        state (i.e. via :meth:`sge.Room.resume`).
-
+        """
+        Called when the room is resumed with :meth:`sge.Room.resume`
+        after it has already previously been started.  It is always
+        called before any :meth:`sge.Object.event_create` occurring at
+        the same time.
         """
         pass
 
     def event_room_end(self):
-        """Room end event.
-
-        Called when the room ends.  It is always called before any game
-        end events occurring at the same time.
-
+        """
+        Called when another room is started or the game ends while this
+        room is the current room.  It is always called before any
+        :meth:`sge.Game.event_game_end` occurring at the same time.
         """
         pass
 
     def event_step(self, time_passed, delta_mult):
-        """Room step event.
-
+        """
         See the documentation for :meth:`sge.Game.event_step` for more
         information.
-
         """
         pass
 
     def event_alarm(self, alarm_id):
-        """Alarm event.
-
-        See the documentation for :meth:`sge.Game.event_alarm` for more
+        """
+        See the documentation for :meth:`sge.Room.alarms` for more
         information.
-
         """
         pass
 
     def event_key_press(self, key, char):
-        """Key press event.
-
+        """
         See the documentation for :class:`sge.input.KeyPress` for more
         information.
-
         """
         pass
 
     def event_key_release(self, key):
-        """Key release event.
-
+        """
         See the documentation for :class:`sge.input.KeyRelease` for more
         information.
-
         """
         pass
 
     def event_mouse_move(self, x, y):
-        """Mouse move event.
-
+        """
         See the documentation for :class:`sge.input.MouseMove` for more
         information.
-
         """
         pass
 
@@ -658,273 +636,186 @@ class Room(object):
         pass
 
     def event_mouse_button_release(self, button):
-        """Mouse button release event.
-
+        """
         See the documentation for :class:`sge.input.MouseButtonRelease`
         for more information.
-
         """
         pass
 
     def event_joystick_axis_move(self, js_name, js_id, axis, value):
-        """Joystick axis move event.
-
+        """
         See the documentation for :class:`sge.input.JoystickAxisMove`
         for more information.
-
         """
         pass
 
     def event_joystick_hat_move(self, js_name, js_id, hat, x, y):
-        """Joystick hat move event.
-
+        """
         See the documentation for :class:`sge.input.JoystickHatMove` for
         more information.
-
         """
         pass
 
     def event_joystick_trackball_move(self, js_name, js_id, ball, x, y):
-        """Joystick trackball move event.
-
+        """
         See the documentation for
         :class:`sge.input.JoystickTrackballMove` for more information.
-
         """
         pass
 
     def event_joystick_button_press(self, js_name, js_id, button):
-        """Joystick button press event.
-
+        """
         See the documentation for :class:`sge.input.JoystickButtonPress`
         for more information.
-
         """
         pass
 
     def event_joystick_button_release(self, js_name, js_id, button):
-        """Joystick button release event.
-
+        """
         See the documentation for
         :class:`sge.input.JoystickButtonRelease` for more information.
-
         """
         pass
 
     def event_gain_keyboard_focus(self):
-        """Gain keyboard focus event.
-
+        """
         See the documentation for :class:`sge.input.KeyboardFocusGain`
         for more information.
-
         """
         pass
 
     def event_lose_keyboard_focus(self):
-        """Lose keyboard focus event.
-
+        """
         See the documentation for :class:`sge.input.KeyboardFocusLose`
         for more information.
-
         """
         pass
 
     def event_gain_mouse_focus(self):
-        """Gain mouse focus event.
-
+        """
         See the documentation for :class:`sge.input.MouseFocusGain` for
         more information.
-
         """
         pass
 
     def event_lose_mouse_focus(self):
-        """Lose mouse focus event.
-
+        """
         See the documentation for :class:`sge.input.MouseFocusLose` for
         more information.
-
         """
         pass
 
     def event_close(self):
-        """Close event.
-
+        """
         This is always called before any :meth:`sge.Game.event_close`
         occurring at the same time.
 
         See the documentation for :class:`sge.input.QuitRequest` for
         more information.
-
         """
         pass
 
     def event_paused_key_press(self, key, char):
-        """Key press event when paused.
-
+        """
         See the documentation for :class:`sge.input.KeyPress` for more
         information.
-
         """
         pass
 
     def event_paused_key_release(self, key):
-        """Key release event when paused.
-
+        """
         See the documentation for :class:`sge.input.KeyRelease` for more
         information.
-
         """
         pass
 
     def event_paused_mouse_move(self, x, y):
-        """Mouse move event when paused.
-
+        """
         See the documentation for :class:`sge.input.MouseMove` for more
         information.
-
         """
         pass
 
     def event_paused_mouse_button_press(self, button):
-        """Mouse button press event when paused.
-
+        """
         See the documentation for :class:`sge.input.MouseButtonPress`
         for more information.
-
         """
         pass
 
     def event_paused_mouse_button_release(self, button):
-        """Mouse button release event when paused.
-
+        """
         See the documentation for :class:`sge.input.MouseButtonRelease`
         for more information.
-
         """
         pass
 
     def event_paused_joystick_axis_move(self, js_name, js_id, axis, value):
-        """Joystick axis move event when paused.
-
+        """
         See the documentation for :class:`sge.input.JoystickAxisMove`
         for more information.
-
         """
         pass
 
     def event_paused_joystick_hat_move(self, js_name, js_id, hat, x, y):
-        """Joystick hat move event when paused.
-
+        """
         See the documentation for :class:`sge.input.JoystickHatMove` for
         more information.
-
         """
         pass
 
     def event_paused_joystick_trackball_move(self, js_name, js_id, ball, x, y):
-        """Joystick trackball move event when paused.
-
+        """
         See the documentation for
         :class:`sge.input.JoystickTrackballMove` for more information.
-
         """
         pass
 
     def event_paused_joystick_button_press(self, js_name, js_id, button):
-        """Joystick button press event when paused.
-
+        """
         See the documentation for :class:`sge.input.JoystickButtonPress`
         for more information.
-
         """
         pass
 
     def event_paused_joystick_button_release(self, js_name, js_id, button):
-        """Joystick button release event when paused.
-
+        """
         See the documentation for
         :class:`sge.input.JoystickButtonRelease` for more information.
-
         """
         pass
 
     def event_paused_gain_keyboard_focus(self):
-        """Gain keyboard focus event when paused.
-
+        """
         See the documentation for :class:`sge.input.KeyboardFocusGain`
         for more information.
-
         """
         pass
 
     def event_paused_lose_keyboard_focus(self):
-        """Lose keyboard focus event when paused.
-
+        """
         See the documentation for :class:`sge.input.KeyboardFocusLose`
         for more information.
-
         """
         pass
 
     def event_paused_gain_mouse_focus(self):
-        """Gain mouse focus event when paused.
-
+        """
         See the documentation for :class:`sge.input.MouseFocusGain` for
         more information.
-
         """
         pass
 
     def event_paused_lose_mouse_focus(self):
-        """Lose mouse focus event when paused.
-
+        """
         See the documentation for :class:`sge.input.MouseFocusLose` for
         more information.
-
         """
         pass
 
     def event_paused_close(self):
-        """Close event when paused.
-
+        """
         See the documentation for :meth:`sge.Room.event_close` for more
         information.
-
         """
         pass
-
-    def _reset(self):
-        # Reset the room to its original state.
-        self._started = False
-        self.width = self._start_width
-        self.height = self._start_height
-        self.views = self._start_views
-        self.background = self._start_background
-        self.objects = self._start_objects[:]
-
-        for view in self.views:
-            view._reset()
-            view.x = self._view_start_x[id(view)]
-            view.y = self._view_start_y[id(view)]
-            view.xport = self._view_start_xport[id(view)]
-            view.yport = self._view_start_yport[id(view)]
-            view.width = self._view_start_width[id(view)]
-            view.height = self._view_start_height[id(view)]
-
-        for obj in self.objects:
-            if obj is not sge.game.mouse:
-                obj.x = self._object_start_x[obj.id]
-                obj.y = self._object_start_y[obj.id]
-
-            obj.z = self._object_start_z[obj.id]
-            obj.sprite = self._object_start_sprite[obj.id]
-            obj.visible = self._object_start_visible[obj.id]
-            obj.checks_collisions = self._object_start_checks_collisions[obj.id]
-            obj.tangible = self._object_start_tangible[obj.id]
-            obj.bbox_x = self._object_start_bbox_x[obj.id]
-            obj.bbox_y = self._object_start_bbox_y[obj.id]
-            obj.bbox_width = self._object_start_bbox_width[obj.id]
-            obj.bbox_height = self._object_start_bbox_height[obj.id]
-            obj.collision_ellipse = self._object_start_collision_ellipse[obj.id]
-            obj.collision_precise = self._object_start_collision_precise[obj.id]
