@@ -779,8 +779,8 @@ def r_update_pixelate(self, complete):
     smooth = sge.game.scale_smooth
     sge.game.scale_smooth = False
 
-    if complete < 0.5:
-        complete *= 2
+    if complete < 0.8:
+        complete *= 1.25
         swidth = max(1, w * (1 - complete))
         sheight = max(1, h * (1 - complete))
         transition_sprite.width = swidth
@@ -788,7 +788,7 @@ def r_update_pixelate(self, complete):
         transition_sprite.width = w
         transition_sprite.height = h
     else:
-        diff = (complete - self.rd["t_complete_last"]) * 2
+        diff = (complete - self.rd["t_complete_last"]) * 5
         c = sge.Color((0, 0, 0, int(round(diff * 255))))
         eraser = sge.Sprite(width=w, height=w)
         eraser.draw_rectangle(0, 0, w, h, c)
